@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -252,8 +258,6 @@ class SegmentTree
 public:
     SegmentTree(QPathSegments &segments);
 
-    QRectF boundingRect() const;
-
     void produceIntersections(int segment);
 
 private:
@@ -304,12 +308,6 @@ SegmentTree::SegmentTree(QPathSegments &segments)
     m_tree[0] = root;
 }
 
-QRectF SegmentTree::boundingRect() const
-{
-    return QRectF(QPointF(m_bounds.x1, m_bounds.y1),
-                  QPointF(m_bounds.x2, m_bounds.y2));
-}
-
 static inline qreal coordinate(const QPointF &pos, int axis)
 {
     return axis == 0 ? pos.x() : pos.y();
@@ -548,11 +546,11 @@ void SegmentTree::produceIntersectionsLeaf(const TreeNode &node, int segment)
 
         for (int k = 0; k < m_intersections.size(); ++k) {
             QPathSegments::Intersection i_isect, j_isect;
-            i_isect.vertex = j_isect.vertex = m_segments.addPoint(m_intersections.at(k).pos);
-
             i_isect.t = m_intersections.at(k).alphaA;
             j_isect.t = m_intersections.at(k).alphaB;
 
+            i_isect.vertex = j_isect.vertex = m_segments.addPoint(m_intersections.at(k).pos);
+
             i_isect.next = 0;
             j_isect.next = 0;
 
@@ -1650,7 +1648,7 @@ bool QPathClipper::doClip(QWingedEdge &list, ClipperMode mode)
 #ifdef QDEBUG_CLIPPER
     printf("sorted y coords:\n");
     for (int i = 0; i < y_coords.size(); ++i) {
-        printf("%.9f\n", y_coords[i]);
+        printf("%.9f\n", y_coords.at(i));
     }
 #endif
 
@@ -1688,23 +1686,23 @@ bool QPathClipper::doClip(QWingedEdge &list, ClipperMode mode)
             QPathVertex *b = list.vertex(edge->second);
 
             // FIXME: this can be optimized by using binary search
-            const int first = qFuzzyFind(y_coords.begin(), y_coords.end(), qMin(a->y, b->y)) - y_coords.begin();
-            const int last = qFuzzyFind(y_coords.begin() + first, y_coords.end(), qMax(a->y, b->y)) - y_coords.begin();
+            const int first = qFuzzyFind(y_coords.cbegin(), y_coords.cend(), qMin(a->y, b->y)) - y_coords.cbegin();
+            const int last = qFuzzyFind(y_coords.cbegin() + first, y_coords.cend(), qMax(a->y, b->y)) - y_coords.cbegin();
 
             Q_ASSERT(first < y_coords.size() - 1);
             Q_ASSERT(last < y_coords.size());
 
-            qreal bestY = 0.5 * (y_coords[first] + y_coords[first+1]);
-            qreal biggestGap = y_coords[first+1] - y_coords[first];
-
+            qreal biggestGap = y_coords.at(first + 1) - y_coords.at(first);
+            int bestIdx = first;
             for (int i = first + 1; i < last; ++i) {
-                qreal gap = y_coords[i+1] - y_coords[i];
+                qreal gap = y_coords.at(i + 1) - y_coords.at(i);
 
                 if (gap > biggestGap) {
-                    bestY = 0.5 * (y_coords[i] + y_coords[i+1]);
+                    bestIdx = i;
                     biggestGap = gap;
                 }
             }
+            const qreal bestY = 0.5 * (y_coords.at(bestIdx) + y_coords.at(bestIdx + 1));
 
 #ifdef QDEBUG_CLIPPER
             printf("y: %.9f, gap: %.9f\n", bestY, biggestGap);
@@ -1755,6 +1753,7 @@ struct QCrossingEdge
         return x < edge.x;
     }
 };
+Q_DECLARE_TYPEINFO(QCrossingEdge, Q_PRIMITIVE_TYPE);
 
 static bool bool_op(bool a, bool b, QPathClipper::Operation op)
 {
@@ -1889,10 +1888,10 @@ bool QPathClipper::handleCrossingEdges(QWingedEdge &list, qreal y, ClipperMode m
 
 namespace {
 
-QList<QPainterPath> toSubpaths(const QPainterPath &path)
+QVector<QPainterPath> toSubpaths(const QPainterPath &path)
 {
 
-    QList<QPainterPath> subpaths;
+    QVector<QPainterPath> subpaths;
     if (path.isEmpty())
         return subpaths;
 
@@ -2092,7 +2091,7 @@ QPainterPath clip(const QPainterPath &path, qreal t)
 
 QPainterPath intersectPath(const QPainterPath &path, const QRectF &rect)
 {
-    QList<QPainterPath> subpaths = toSubpaths(path);
+    QVector<QPainterPath> subpaths = toSubpaths(path);
 
     QPainterPath result;
     result.setFillRule(path.fillRule());
